@@ -17,6 +17,7 @@
 # Refer to the README and COPYING files for full details of the license
 #
 
+from __future__ import absolute_import
 from ..utils import getHostUUID
 from . import expose, ExtraArgsError
 import sys
@@ -33,5 +34,5 @@ def getUUID(*args):
     hostUUID = getHostUUID(False)
     if hostUUID is None:
         raise EnvironmentError('Cannot retrieve host UUID')
-    sys.stdout.write(hostUUID)
+    sys.stdout.write(hostUUID + '\n')
     return 0
