@@ -16,7 +16,7 @@
 #
 # Refer to the README and COPYING files for full details of the license
 #
-from testrunner import VdsmTestCase as TestCaseBase
+from testlib import VdsmTestCase as TestCaseBase
 import testValidation
 import supervdsm
 from vdsm.constants import VDSM_USER
