@@ -3,7 +3,7 @@ if [ -z "$PYTHON_EXE" ]; then
     PYTHON_EXE="/usr/bin/python"
 fi
 
-if [ ! -f ../tests/jsonrpc-tests.server.crt ] || [ ! -f ../tests/jsonrpc-tests.server.csr ] || [ ! -f ../tests/jsonrpc-tests.server.key ]; then
+if [ ! -f ../tests/server.crt ] || [ ! -f ../tests/server.csr ] || [ ! -f ../tests/server.key ] || [ ! -f ../tests/other.crt ] || [ ! -f ../tests/other.csr ] || [ ! -f ../tests/other.key ]; then
     ../tests/makecert.sh
 fi
 
