@@ -17,7 +17,7 @@
 # Refer to the README and COPYING files for full details of the license
 #
 
-from testrunner import VdsmTestCase as TestCaseBase
+from testlib import VdsmTestCase as TestCaseBase
 from rpc import vdsmapi
 
 
