@@ -17,24 +17,25 @@
 # Refer to the README and COPYING files for full details of the license
 #
 
+from __future__ import absolute_import
 import logging
 
 from vdsm import netinfo
 from vdsm import ipwrapper
 from vdsm.constants import EXT_BRCTL
-from vdsm.ipwrapper import routeAdd
-from vdsm.ipwrapper import routeDel
-from vdsm.ipwrapper import ruleAdd
-from vdsm.ipwrapper import ruleDel
+from vdsm.ipwrapper import routeAdd, routeDel, ruleAdd, ruleDel, IPRoute2Error
 from vdsm.netconfpersistence import RunningConfig
+from vdsm import sysctl
 from vdsm.utils import CommandPath
 from vdsm.utils import execCmd
 
-from . import Configurator, getEthtoolOpts, libvirt
+from . import (Configurator, runDhclient, getEthtoolOpts, libvirt,
+               wait_for_device)
 from .dhclient import DhcpClient
 from ..errors import ConfigNetworkError, ERR_FAILED_IFUP, ERR_FAILED_IFDOWN
 from ..models import Nic
 from ..sourceroute import DynamicSourceRoute
+from ..utils import remove_custom_bond_option
 
 _BRIDGING_OPT_PATH = '/sys/class/net/%s/bridge/%s'
 _ETHTOOL_BINARY = CommandPath(
@@ -43,6 +44,12 @@ _ETHTOOL_BINARY = CommandPath(
     '/sbin/ethtool',  # EL6, ubuntu and Debian
     '/usr/bin/ethtool',  # Arch
 )
+_BRCTL_DEV_EXISTS = ("device %s already exists; can't create bridge with the "
+                     "same name")
+
+
+def is_available():
+    return True
 
 
 class Iproute2(Configurator):
@@ -68,6 +75,10 @@ class Iproute2(Configurator):
             self.configApplier.addBridgePort(bridge)
         DynamicSourceRoute.addInterfaceTracking(bridge)
         self.configApplier.setIfaceConfigAndUp(bridge)
+        if not bridge.ipv6.address and not bridge.ipv6.ipv6autoconf and (
+                not bridge.ipv6.dhcpv6):
+            wait_for_device(bridge.name)
+            sysctl.disable_ipv6(bridge.name)
         self._addSourceRoute(bridge)
         if 'custom' in opts and 'bridge_opts' in opts['custom']:
             self.configApplier._setBridgeOpts(bridge,
@@ -207,7 +218,15 @@ class Iproute2(Configurator):
     @staticmethod
     def removeSourceRoute(routes, rules, device):
         for route in routes:
-            routeDel(route)
+            try:
+                routeDel(route)
+            except IPRoute2Error as e:
+                if 'No such process' in e.message[0]:
+                    # The kernel or dhclient has won the race and removed the
+                    # route already. We have yet to remove routing rules.
+                    pass
+                else:
+                    raise
 
         for rule in rules:
             ruleDel(rule)
@@ -216,24 +235,25 @@ class Iproute2(Configurator):
 class ConfigApplier(object):
 
     def _setIpConfig(self, iface):
-        ipConfig = iface.ipConfig
-        if ipConfig.ipaddr or ipConfig.ipv6addr:
+        ipv4 = iface.ipv4
+        ipv6 = iface.ipv6
+        if ipv4.address or ipv6.address:
             self.removeIpConfig(iface)
-        if ipConfig.ipaddr:
-            ipwrapper.addrAdd(iface.name, ipConfig.ipaddr,
-                              ipConfig.netmask)
-            if ipConfig.gateway and ipConfig.defaultRoute:
-                ipwrapper.routeAdd(['default', 'via', ipConfig.gateway])
-        if ipConfig.ipv6addr:
-            ipv6addr, ipv6netmask = ipConfig.ipv6addr.split('/')
+        if ipv4.address:
+            ipwrapper.addrAdd(iface.name, ipv4.address,
+                              ipv4.netmask)
+            if ipv4.gateway and ipv4.defaultRoute:
+                ipwrapper.routeAdd(['default', 'via', ipv4.gateway])
+        if ipv6.address:
+            ipv6addr, ipv6netmask = ipv6.address.split('/')
             ipwrapper.addrAdd(iface.name, ipv6addr, ipv6netmask, family=6)
-            if ipConfig.ipv6gateway:
-                ipwrapper.routeAdd(['default', 'via', ipConfig.ipv6gateway],
+            if ipv6.gateway:
+                ipwrapper.routeAdd(['default', 'via', ipv6.gateway],
                                    dev=iface.name, family=6)
-        if ipConfig.ipv6autoconf is not None:
+        if ipv6.ipv6autoconf is not None:
             with open('/proc/sys/net/ipv6/conf/%s/autoconf' % iface.name,
                       'w') as ipv6_autoconf:
-                ipv6_autoconf.write('1' if ipConfig.ipv6autoconf else '0')
+                ipv6_autoconf.write('1' if ipv6.ipv6autoconf else '0')
 
     def removeIpConfig(self, iface):
         ipwrapper.addrFlush(iface.name)
@@ -246,11 +266,10 @@ class ConfigApplier(object):
 
     def ifup(self, iface):
         ipwrapper.linkSet(iface.name, ['up'])
-        if iface.ipConfig.bootproto == 'dhcp':
-            dhclient = DhcpClient(iface.name)
-            rc = dhclient.start(iface.ipConfig.async)
-            if not iface.ipConfig.async and rc:
-                raise ConfigNetworkError(ERR_FAILED_IFUP, 'dhclient failed')
+        if iface.ipv4.bootproto == 'dhcp':
+            runDhclient(iface, 4, iface.ipv4.defaultRoute)
+        if iface.ipv6.dhcpv6:
+            runDhclient(iface, 6, iface.ipv6.defaultRoute)
 
     def ifdown(self, iface):
         ipwrapper.linkSet(iface.name, ['down'])
@@ -258,7 +277,7 @@ class ConfigApplier(object):
         dhclient.shutdown()
 
     def setIfaceConfigAndUp(self, iface):
-        if iface.ip:
+        if iface.ipv4 or iface.ipv6:
             self._setIpConfig(iface)
         if iface.mtu:
             self.setIfaceMtu(iface.name, iface.mtu)
@@ -267,7 +286,10 @@ class ConfigApplier(object):
     def addBridge(self, bridge):
         rc, _, err = execCmd([EXT_BRCTL, 'addbr', bridge.name])
         if rc != 0:
-            raise ConfigNetworkError(ERR_FAILED_IFUP, err)
+            err_used_bridge = (_BRCTL_DEV_EXISTS % bridge.name == err[0]
+                               if err else False)
+            if not err_used_bridge:
+                raise ConfigNetworkError(ERR_FAILED_IFUP, err)
         if bridge.stp:
             with open(netinfo.BRIDGING_OPT %
                       (bridge.name, 'stp_state'), 'w') as bridge_stp:
@@ -321,12 +343,14 @@ class ConfigApplier(object):
 
     def addBondOptions(self, bond):
         logging.debug('Add bond options %s', bond.options)
-        for option in bond.options.split():
+        # 'custom' is not a real bond option, it just piggybacks custom values
+        options = remove_custom_bond_option(bond.options)
+        for option in options:
             key, value = option.split('=')
             with open(netinfo.BONDING_OPT % (bond.name, key), 'w') as f:
                 f.write(value)
 
-    def createLibvirtNetwork(self, network, bridged, iface):
+    def createLibvirtNetwork(self, network, bridged=True, iface=None):
         netXml = libvirt.createNetworkDef(network, bridged, iface)
         libvirt.createNetwork(netXml)
 
