@@ -66,6 +66,10 @@ getProcPool = oop.getGlobalProcPool
 def validateDirAccess(dirPath):
     try:
         getProcPool().fileUtils.validateAccess(dirPath)
+        supervdsm.getProxy().validateAccess(
+            constants.VDSM_USER,
+            (constants.VDSM_GROUP,), dirPath,
+            (os.R_OK | os.W_OK | os.X_OK))
         supervdsm.getProxy().validateAccess(
             constants.QEMU_PROCESS_USER,
             (constants.DISKIMAGE_GROUP, constants.METADATA_GROUP), dirPath,
@@ -145,8 +149,8 @@ FileSDMetadata = lambda metafile: DictValidator(
     PersistentDict(FileMetadataRW(metafile)), FILE_SD_MD_FIELDS)
 
 
-class FileStorageDomain(sd.StorageDomain):
-    def __init__(self, domainPath):
+class FileStorageDomainManifest(sd.StorageDomainManifest):
+    def __init__(self, domainPath, metadata=None):
         # Using glob might look like the simplest thing to do but it isn't
         # If one of the mounts is stuck it'll cause the entire glob to fail
         # and you wouldn't be able to access any domain
@@ -155,16 +159,44 @@ class FileStorageDomain(sd.StorageDomain):
         self.remotePath = os.path.basename(self.mountpoint)
         self.metafile = os.path.join(domainPath, sd.DOMAIN_META_DATA,
                                      sd.METADATA)
-
         sdUUID = os.path.basename(domainPath)
-        validateFileSystemFeatures(sdUUID, self.mountpoint)
-
-        metadata = FileSDMetadata(self.metafile)
         domaindir = os.path.join(self.mountpoint, sdUUID)
-        sd.StorageDomain.__init__(self, sdUUID, domaindir, metadata)
+        sd.StorageDomainManifest.__init__(self, sdUUID, domaindir)
 
+        if metadata is None:
+            metadata = FileSDMetadata(self.metafile)
+        self.replaceMetadata(metadata)
         if not self.oop.fileUtils.pathExists(self.metafile):
-            raise se.StorageDomainMetadataNotFound(sdUUID, self.metafile)
+            raise se.StorageDomainMetadataNotFound(self.sdUUID, self.metafile)
+
+    def getReadDelay(self):
+        stats = misc.readspeed(self.metafile, 4096)
+        return stats['seconds']
+
+    def getVSize(self, imgUUID, volUUID):
+        """ Returns file volume size in bytes. """
+        volPath = os.path.join(self.mountpoint, self.sdUUID, 'images',
+                               imgUUID, volUUID)
+        return self.oop.os.stat(volPath).st_size
+
+    def getLeasesFilePath(self):
+        return os.path.join(self.getMDPath(), sd.LEASES)
+
+    def getIdsFilePath(self):
+        return os.path.join(self.getMDPath(), sd.IDS)
+
+
+class FileStorageDomain(sd.StorageDomain):
+    manifestClass = FileStorageDomainManifest
+
+    def __init__(self, domainPath):
+        manifest = self.manifestClass(domainPath)
+
+        # We perform validation here since filesystem features are relevant to
+        # construction of an actual Storage Domain.  Direct users of
+        # FileStorageDomainManifest should call this explicitly if required.
+        validateFileSystemFeatures(manifest.sdUUID, manifest.mountpoint)
+        sd.StorageDomain.__init__(self, manifest)
         self.imageGarbageCollector()
         self._registerResourceNamespaces()
 
@@ -258,10 +290,6 @@ class FileStorageDomain(sd.StorageDomain):
             REMOTE_PATH: remotePath
         })
 
-    def getReadDelay(self):
-        stats = misc.readspeed(self.metafile, 4096)
-        return stats['seconds']
-
     def getFileList(self, pattern, caseSensitive):
         """
         Returns a list of all files in the domain filtered according to
@@ -303,19 +331,6 @@ class FileStorageDomain(sd.StorageDomain):
         """
         return fileVolume.FileVolume
 
-    def volumeExists(self, imgPath, volUUID):
-        """
-        Return True if the volume volUUID exists
-        """
-        volPath = os.path.join(imgPath, volUUID)
-        return self.oop.fileUtils.pathExists(volPath)
-
-    def getVSize(self, imgUUID, volUUID):
-        """ Returns file volume size in bytes. """
-        volPath = os.path.join(self.mountpoint, self.sdUUID, 'images',
-                               imgUUID, volUUID)
-        return self.oop.os.stat(volPath).st_size
-
     def getVAllocSize(self, imgUUID, volUUID):
         """ Returns file volume allocated size in bytes. """
         volPath = os.path.join(self.mountpoint, self.sdUUID, 'images',
@@ -371,7 +386,7 @@ class FileStorageDomain(sd.StorageDomain):
     def deleteImage(self, sdUUID, imgUUID, volsImgs):
         currImgDir = self.getImagePath(imgUUID)
         dirName, baseName = os.path.split(currImgDir)
-        toDelDir = os.tempnam(dirName, sd.REMOVED_IMAGE_PREFIX + baseName)
+        toDelDir = os.path.join(dirName, sd.REMOVED_IMAGE_PREFIX + baseName)
         self.log.debug("Renaming dir %s to %s", currImgDir, toDelDir)
         try:
             self.oop.os.rename(currImgDir, toDelDir)
@@ -544,7 +559,7 @@ class FileStorageDomain(sd.StorageDomain):
         return True
 
     def getRemotePath(self):
-        return self.remotePath
+        return self._manifest.remotePath
 
     def getRealPath(self):
         """
@@ -663,7 +678,7 @@ class FileStorageDomain(sd.StorageDomain):
                 self.oop.utils.forceLink(tVol, tLink)
 
 
-def getMountsList(pattern="*"):
+def _getMountsList(pattern="*"):
     fileDomPattern = os.path.join(
         sd.StorageDomain.storage_repository, sd.DOMAIN_MNT_POINT,
         pattern)
@@ -686,7 +701,7 @@ def getMountsList(pattern="*"):
 def scanDomains(pattern="*"):
     log = logging.getLogger("Storage.scanDomains")
 
-    mntList = getMountsList(pattern)
+    mntList = _getMountsList(pattern)
 
     def collectMetaFiles(possibleDomain):
         try:
