@@ -134,28 +134,28 @@ def createdir(dirPath, mode=None):
     If already exists check that permissions are as requested.
     """
     if mode is not None:
-        mode |= stat.S_IFDIR
+        mode = stat.S_IMODE(mode)
         params = (dirPath, mode)
     else:
         params = (dirPath,)
 
-    log.debug("Creating directory: %s", dirPath)
+    log.debug("Creating directory: %s mode: %s", dirPath,
+              mode if mode is None else oct(mode))
     try:
         os.makedirs(*params)
     except OSError as e:
         if e.errno != errno.EEXIST:
             raise
-        else:
-            log.warning("Dir %s already exists", dirPath)
-            if mode is not None:
-                statinfo = os.stat(dirPath)
-                curMode = statinfo[stat.ST_MODE]
-                if curMode != mode:
-                    raise OSError(errno.EPERM,
-                                  ("Existing %s permissions %s are not as "
-                                   "requested %s") % (dirPath,
-                                                      oct(curMode),
-                                                      oct(mode)))
+        statinfo = os.stat(dirPath)
+        if not stat.S_ISDIR(statinfo.st_mode):
+            raise OSError(errno.ENOTDIR, "Not a directory %s" % dirPath)
+        log.warning("Dir %s already exists", dirPath)
+        if mode is not None:
+            curMode = stat.S_IMODE(statinfo.st_mode)
+            if curMode != mode:
+                raise OSError(errno.EPERM,
+                              ("Existing %s permissions %o are not as "
+                               "requested %o") % (dirPath, curMode, mode))
 
 
 def resolveUid(user):
@@ -375,7 +375,7 @@ def copyUserModeToGroup(path):
 
 
 def padToBlockSize(path):
-    with file(path, 'a') as f:
+    with open(path, 'a') as f:
         size = os.fstat(f.fileno()).st_size
         newSize = 512 * ((size + 511) / 512)
         log.debug("Truncating file %s to %d bytes", path, newSize)
