@@ -18,6 +18,7 @@
 # Refer to the README and COPYING files for full details of the license
 #
 
+import array
 import logging
 import time
 import socket
@@ -30,18 +31,20 @@ import supervdsm
 
 from . import vmstatus
 
-_MAX_SUPPORTED_API_VERSION = 1
+_MAX_SUPPORTED_API_VERSION = 2
 _IMPLICIT_API_VERSION_ZERO = 0
 
 _MESSAGE_API_VERSION_LOOKUP = {
     'set-number-of-cpus': 1}
 
-__REPLACEMENT_CHAR = u'\ufffd'
-__RESTRICTED_CHARS = set(range(8 + 1)). \
-    union(set(range(0xB, 0xC + 1))). \
-    union(set(range(0xE, 0x1F + 1))). \
-    union(set(range(0x7F, 0x84 + 1))). \
-    union(set(range(0x86, 0x9F + 1)))
+_REPLACEMENT_CHAR = u'\ufffd'
+_RESTRICTED_CHARS = frozenset(unichr(c) for c in
+                              list(range(8 + 1)) +
+                              list(range(0xB, 0xC + 1)) +
+                              list(range(0xE, 0x1F + 1)) +
+                              list(range(0x7F, 0x84 + 1)) +
+                              list(range(0x86, 0x9F + 1)) +
+                              [0xFFFE, 0xFFFF])
 
 
 def _filterXmlChars(u):
@@ -51,7 +54,7 @@ def _filterXmlChars(u):
 
     "Char" is defined as any unicode character except the surrogate blocks,
     \ufffe and \uffff.
-    "RestrictedChar" is defiend as the code points in __RESTRICTED_CHARS above
+    "RestrictedChar" is defiend as the code points in _RESTRICTED_CHARS above
 
     It's a little hard to follow, but the upshot is an XML document
     must contain only characters in Char that are not in
@@ -65,19 +68,12 @@ def _filterXmlChars(u):
     if not isinstance(u, unicode):
         raise TypeError
 
-    def filterXmlChar(c):
-        if ord(c) > 0x10ffff:
-            return __REPLACEMENT_CHAR  # Outside Unicode range
-        elif unicodedata.category(c) == 'Cs':
-            return __REPLACEMENT_CHAR  # Surrogate pair code point
-        elif (ord(c) == 0xFFFE) or (ord(c) == 0xFFFF):
-            return __REPLACEMENT_CHAR  # Specifically excluded code points
-        elif ord(c) in __RESTRICTED_CHARS:
-            return __REPLACEMENT_CHAR  # Restricted character
-        else:
-            return c
-
-    return ''.join(filterXmlChar(c) for c in u)
+    chars = array.array('u', u)
+    for i, c in enumerate(chars):
+        if (c > u'\U00010fff' or unicodedata.category(c) == 'Cs'
+                or c in _RESTRICTED_CHARS):
+            chars[i] = _REPLACEMENT_CHAR
+    return chars.tounicode()
 
 
 def _filterObject(obj):
@@ -109,19 +105,19 @@ class GuestAgentUnsupportedMessage(Exception):
         Exception.__init__(self, message)
 
 
-class GuestAgent ():
+class GuestAgent(object):
     MAX_MESSAGE_SIZE = 2 ** 20  # 1 MiB for now
 
-    def __init__(self, socketName, channelListener, log, user='Unknown',
-                 ips='', connect=True):
+    def __init__(self, socketName, channelListener, log, onStatusChange,
+                 user='Unknown', ips=''):
         self.effectiveApiVersion = _IMPLICIT_API_VERSION_ZERO
+        self._onStatusChange = onStatusChange
         self.log = log
         self._socketName = socketName
         self._sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
         self._stopped = True
-        self.guestStatus = None
+        self._status = None
         self.guestDiskMapping = {}
-        self.diskMappingHash = 0
         self.guestInfo = {
             'username': user,
             'memUsage': 0,
@@ -136,18 +132,39 @@ class GuestAgent ():
         self._agentTimestamp = 0
         self._channelListener = channelListener
         self._messageState = MessageState.NORMAL
-        if connect:
-            try:
-                self._prepare_socket()
-            except:
-                self.log.error("Failed to prepare vmchannel", exc_info=True)
-            else:
-                self._channelListener.register(
-                    self._create,
-                    self._connect,
-                    self._onChannelRead,
-                    self._onChannelTimeout,
-                    self)
+
+    @property
+    def guestStatus(self):
+        return self._status
+
+    @guestStatus.setter
+    def guestStatus(self, value):
+        oldValue = self._status
+        self._status = value
+        if oldValue != value and self._onStatusChange:
+            self._onStatusChange()
+
+    @property
+    def guestDiskMapping(self):
+        return self._guestDiskMapping
+
+    @guestDiskMapping.setter
+    def guestDiskMapping(self, value):
+        self._guestDiskMapping = value
+        self._diskMappingHash = hash(json.dumps(value, sort_keys=True))
+
+    @property
+    def diskMappingHash(self):
+        return self._diskMappingHash
+
+    def connect(self):
+        self._prepare_socket()
+        self._channelListener.register(
+            self._create,
+            self._connect,
+            self._onChannelRead,
+            self._onChannelTimeout,
+            self)
 
     def _handleAPIVersion(self, version):
         """ Handles the API version value from the heartbeat
@@ -229,10 +246,7 @@ class GuestAgent ():
 
     def _handleMessage(self, message, args):
         self.log.log(logging.TRACE, "Guest's message %s: %s", message, args)
-        if self.guestStatus is None:
-            self.guestStatus = vmstatus.UP
         if message == 'heartbeat':
-            self.guestStatus = vmstatus.UP
             self.guestInfo['memUsage'] = int(args['free-ram'])
             # ovirt-guest-agent reports the following fields in 'memory-stat':
             # 'mem_total', 'mem_free', 'mem_unused', 'swap_in', 'swap_out',
@@ -253,10 +267,17 @@ class GuestAgent ():
                 # downgrade of the guest agent)
                 self.log.debug("API versioning no longer reported by guest.")
                 self.effectiveApiVersion = _IMPLICIT_API_VERSION_ZERO
+            # Only change the state AFTER all data of the heartbeat has been
+            # consumed
+            self.guestStatus = vmstatus.UP
         elif message == 'host-name':
             self.guestInfo['guestName'] = args['name']
         elif message == 'os-version':
             self.guestInfo['guestOs'] = args['version']
+        elif message == 'os-info':
+            self.guestInfo['guestOsInfo'] = args
+        elif message == 'timezone':
+            self.guestInfo['guestTimezone'] = args
         elif message == 'network-interfaces':
             interfaces = []
             old_ips = ''
@@ -306,8 +327,6 @@ class GuestAgent ():
                 disks.append(disk)
             self.guestInfo['disksUsage'] = disks
             self.guestDiskMapping = args.get('mapping', {})
-            self.diskMappingHash = hash(json.dumps(self.guestDiskMapping,
-                                                   sort_keys=True))
         elif message == 'number-of-cpus':
             self.guestInfo['guestCPUCount'] = int(args['count'])
         else:
@@ -356,7 +375,7 @@ class GuestAgent ():
             self.log.debug("desktopLock called")
             self._forward("lock-screen")
         except:
-            self.log.error("desktopLock failed", exc_info=True)
+            self.log.exception("desktopLock failed")
 
     def desktopLogin(self, domain, user, password):
         try:
@@ -366,16 +385,16 @@ class GuestAgent ():
             else:
                 username = user
             self._forward('login', {'username': username,
-                                    "password": password})
+                                    "password": password.value})
         except:
-            self.log.error("desktopLogin failed", exc_info=True)
+            self.log.exception("desktopLogin failed")
 
     def desktopLogoff(self, force):
         try:
             self.log.debug("desktopLogoff called")
             self._forward('log-off')
         except:
-            self.log.error("desktopLogoff failed", exc_info=True)
+            self.log.exception("desktopLogoff failed")
 
     def desktopShutdown(self, timeout, msg, reboot):
         try:
@@ -383,14 +402,14 @@ class GuestAgent ():
             self._forward('shutdown', {'timeout': timeout, 'message': msg,
                                        'reboot': str(reboot)})
         except:
-            self.log.error("desktopShutdown failed", exc_info=True)
+            self.log.exception("desktopShutdown failed")
 
     def sendHcCmdToDesktop(self, cmd):
         try:
             self.log.debug("sendHcCmdToDesktop('%s')" % (cmd))
             self._forward(str(cmd))
         except:
-            self.log.error("sendHcCmdToDesktop failed", exc_info=True)
+            self.log.exception("sendHcCmdToDesktop failed")
 
     def setNumberOfCPUs(self, count):
         self.log.debug("setNumberOfCPUs('%d') called", count)
