@@ -23,7 +23,7 @@ import time
 import select
 import logging
 
-from storage.misc import NoIntrPoll
+from vdsm.utils import NoIntrPoll
 
 # How many times a reconnect should be performed before a cooldown will be
 # applied
@@ -51,7 +51,7 @@ class Listener(threading.Thread):
         """ Handle an epoll event occurred on a specific file descriptor. """
         reconnect = False
         if (event & (select.EPOLLHUP | select.EPOLLERR)):
-            self.log.info("Received %.08X on fileno %d", event, fileno)
+            self.log.debug("Received %.08X on fileno %d", event, fileno)
             if fileno in self._channels:
                 reconnect = True
             else:
@@ -60,6 +60,7 @@ class Listener(threading.Thread):
         elif (event & select.EPOLLIN):
             obj = self._channels.get(fileno, None)
             if obj:
+                obj['timeout_seen'] = False
                 obj['reconnects'] = 0
                 try:
                     if obj['read_cb'](obj['opaque']):
@@ -77,6 +78,7 @@ class Listener(threading.Thread):
 
     def _prepare_reconnect(self, fileno):
             obj = self._channels.pop(fileno)
+            obj['timeout_seen'] = False
             try:
                 fileno = obj['create_cb'](obj['opaque'])
             except:
@@ -93,7 +95,9 @@ class Listener(threading.Thread):
         now = time.time()
         for (fileno, obj) in self._channels.items():
             if (now - obj['read_time']) >= self._timeout:
-                self.log.debug("Timeout on fileno %d.", fileno)
+                if not obj.get('timeout_seen', False):
+                    self.log.debug("Timeout on fileno %d.", fileno)
+                    obj['timeout_seen'] = True
                 try:
                     obj['timeout_cb'](obj['opaque'])
                     obj['read_time'] = now
@@ -130,7 +134,6 @@ class Listener(threading.Thread):
         """
         now = time.time()
         for (fileno, obj) in self._unconnected.items():
-            self.log.debug("Trying to connect fileno %d.", fileno)
             if obj.get('cooldown'):
                 if (now - obj['cooldown_time']) >= self._timeout:
                     obj['cooldown'] = False
@@ -172,7 +175,7 @@ class Listener(threading.Thread):
 
     def run(self):
         """ The listener thread's function. """
-        self.log.info("Starting VM channels listener thread.")
+        self.log.debug("Starting VM channels listener thread.")
         self._quit = False
         try:
             while not self._quit:
@@ -180,12 +183,13 @@ class Listener(threading.Thread):
         except:
             self.log.exception("Unhandled exception caught in vm channels "
                                "listener thread")
-        self.log.info("VM channels listener thread has ended.")
+        finally:
+            self.log.debug("VM channels listener thread has ended.")
 
     def stop(self):
         """" Stop the listener execution. """
         self._quit = True
-        self.log.info("VM channels listener was stopped.")
+        self.log.debug("VM channels listener was stopped.")
 
     def settimeout(self, seconds):
         """ Set the timeout value (in seconds) for all channels. """
